@@ -1,10 +1,12 @@
+# Copyright (c) ONNX Project Contributors
+#
 # SPDX-License-Identifier: Apache-2.0
 
-import numpy as np  # type: ignore
+import numpy as np
 
 import onnx
-from ..base import Base
-from . import expect
+from onnx.backend.test.case.base import Base
+from onnx.backend.test.case.node import expect
 
 
 def hardswish(x: np.ndarray) -> np.ndarray:
@@ -14,16 +16,14 @@ def hardswish(x: np.ndarray) -> np.ndarray:
 
 
 class HardSwish(Base):
-
     @staticmethod
     def export() -> None:
         node = onnx.helper.make_node(
-            'HardSwish',
-            inputs=['x'],
-            outputs=['y'],
+            "HardSwish",
+            inputs=["x"],
+            outputs=["y"],
         )
         x = np.random.randn(3, 4, 5).astype(np.float32)
         y = hardswish(x)
 
-        expect(node, inputs=[x], outputs=[y],
-               name='test_hardswish')
+        expect(node, inputs=[x], outputs=[y], name="test_hardswish")
