@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 
 import importlib
@@ -7,6 +9,8 @@ from typing import List, Optional
 
 import numpy as np
 
+from onnx import ONNX_ML
+
 all_numeric_dtypes = [
     np.int8,
     np.int16,
@@ -31,6 +35,11 @@ def import_recursive(package: ModuleType) -> None:
     module_location = package.__name__
     for _module_loader, name, ispkg in pkgutil.iter_modules(pkg_dir):
         module_name = f"{module_location}.{name}"  # Module/package
+        if not ONNX_ML and module_name.startswith(
+            "onnx.backend.test.case.node.ai_onnx_ml"
+        ):
+            continue
+
         module = importlib.import_module(module_name)
         if ispkg:
             import_recursive(module)
