@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+#
 # SPDX-License-Identifier: Apache-2.0
 
 from __future__ import annotations
@@ -31,19 +33,19 @@ from urllib.request import urlretrieve
 import numpy as np
 
 import onnx
-from onnx import ModelProto, NodeProto, TypeProto, numpy_helper
+import onnx.reference
+from onnx import ONNX_ML, ModelProto, NodeProto, TypeProto, ValueInfoProto, numpy_helper
 from onnx.backend.base import Backend
-
-from ..case.test_case import TestCase
-from ..loader import load_model_tests
-from .item import TestItem
+from onnx.backend.test.case.test_case import TestCase
+from onnx.backend.test.loader import load_model_tests
+from onnx.backend.test.runner.item import TestItem
 
 
 class BackendIsNotSupposedToImplementIt(unittest.SkipTest):
     pass
 
 
-def retry_excute(times: int) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
+def retry_execute(times: int) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
     assert times >= 1
 
     def wrapper(func: Callable[..., Any]) -> Callable[..., Any]:
@@ -133,7 +135,7 @@ class Runner:
                 for exclude in self._exclude_patterns:
                     if exclude.search(name):
                         item.func = unittest.skip(
-                            f'matched exclude pattern "{exclude.pattern}"'  # noqa: B907
+                            f'matched exclude pattern "{exclude.pattern}"'
                         )(item.func)
                 for xfail in self._xfail_patterns:
                     if xfail.search(name):
@@ -204,7 +206,7 @@ class Runner:
                     )
             else:
                 np.testing.assert_equal(outputs[i].dtype, ref_outputs[i].dtype)
-                if ref_outputs[i].dtype == object:
+                if ref_outputs[i].dtype == object:  # type: ignore[attr-defined]
                     np.testing.assert_array_equal(outputs[i], ref_outputs[i])
                 else:
                     np.testing.assert_allclose(
@@ -212,7 +214,7 @@ class Runner:
                     )
 
     @classmethod
-    @retry_excute(3)
+    @retry_execute(3)
     def download_model(
         cls, model_test: TestCase, model_dir: str, models_dir: str
     ) -> None:
@@ -223,20 +225,14 @@ class Runner:
             download_file.close()
             assert model_test.url
             print(
-                "Start downloading model {} from {}".format(
-                    model_test.model_name, model_test.url
-                )
+                f"Start downloading model {model_test.model_name} from {model_test.url}"
             )
             urlretrieve(model_test.url, download_file.name)
             print("Done")
             with tarfile.open(download_file.name) as t:
                 t.extractall(models_dir)
         except Exception as e:
-            print(
-                "Failed to prepare data for model {}: {}".format(
-                    model_test.model_name, e
-                )
-            )
+            print(f"Failed to prepare data for model {model_test.model_name}: {e}")
             raise
         finally:
             os.remove(download_file.name)
@@ -281,9 +277,7 @@ class Runner:
             device_test_name = f"{test_name}_{device.lower()}"
             if device_test_name in self._test_items[category]:
                 raise ValueError(
-                    'Duplicated test name "{}" in category "{}"'.format(
-                        device_test_name, category
-                    )
+                    f'Duplicated test name "{device_test_name}" in category "{category}"'
                 )
 
             @unittest.skipIf(  # type: ignore
@@ -297,11 +291,7 @@ class Runner:
                 except BackendIsNotSupposedToImplementIt as e:
                     # hacky verbose reporting
                     if "-v" in sys.argv or "--verbose" in sys.argv:
-                        print(
-                            "Test {} is effectively skipped: {}".format(
-                                device_test_name, e
-                            )
-                        )
+                        print(f"Test {device_test_name} is effectively skipped: {e}")
 
             self._test_items[category][device_test_name] = TestItem(
                 device_test_func, report_item
@@ -310,17 +300,76 @@ class Runner:
         for device in devices:
             add_device_test(device)
 
+    @staticmethod
+    def generate_dummy_data(
+        x: ValueInfoProto, seed: int = 0, name: str = "", random: bool = False
+    ) -> np.ndarray:
+        """
+        Generates a random tensor based on the input definition.
+        """
+        if not x.type.tensor_type:
+            raise NotImplementedError(
+                f"Input expected to have tensor type. "
+                f"Unable to generate random data for model {name!r} and input {x}."
+            )
+        if x.type.tensor_type.elem_type != 1:
+            raise NotImplementedError(
+                f"Currently limited to float tensors. "
+                f"Unable to generate random data for model {name!r} and input {x}."
+            )
+        shape = tuple(
+            d.dim_value if d.HasField("dim_value") else 1
+            for d in x.type.tensor_type.shape.dim
+        )
+        if random:
+            gen = np.random.default_rng(seed=seed)
+            return gen.random(shape, np.float32)
+        n = np.prod(shape)
+        return (np.arange(n).reshape(shape) / n).astype(np.float32)
+
     def _add_model_test(self, model_test: TestCase, kind: str) -> None:
         # model is loaded at runtime, note sometimes it could even
         # never loaded if the test skipped
         model_marker: List[Optional[Union[ModelProto, NodeProto]]] = [None]
 
         def run(test_self: Any, device: str) -> None:
-            if model_test.model_dir is None:
-                model_dir = self.prepare_model_data(model_test)
+            if model_test.url is not None and model_test.url.startswith(
+                "onnx/backend/test/data/light/"
+            ):
+                # testing local files
+                model_pb_path = os.path.normpath(
+                    os.path.join(
+                        os.path.dirname(__file__),
+                        "..",
+                        "..",
+                        "..",
+                        "..",
+                        model_test.url,
+                    )
+                )
+                if not os.path.exists(model_pb_path):
+                    raise FileNotFoundError(f"Unable to find model {model_pb_path!r}.")
+                onnx_home = os.path.expanduser(
+                    os.getenv("ONNX_HOME", os.path.join("~", ".onnx"))
+                )
+                models_dir = os.getenv(
+                    "ONNX_MODELS", os.path.join(onnx_home, "models", "light")
+                )
+                model_dir: str = os.path.join(models_dir, model_test.model_name)
+                if not os.path.exists(model_dir):
+                    os.makedirs(model_dir)
+                use_dummy = True
             else:
-                model_dir = model_test.model_dir
-            model_pb_path = os.path.join(model_dir, "model.onnx")
+                if model_test.model_dir is None:
+                    model_dir = self.prepare_model_data(model_test)
+                else:
+                    model_dir = model_test.model_dir
+                model_pb_path = os.path.join(model_dir, "model.onnx")
+                use_dummy = False
+
+            if not ONNX_ML and "ai_onnx_ml" in model_dir:
+                return
+
             model = onnx.load(model_pb_path)
             model_marker[0] = model
             if (
@@ -329,18 +378,76 @@ class Runner:
                 and not self.backend.is_compatible(model)
             ):
                 raise unittest.SkipTest("Not compatible with backend")
+
             prepared_model = self.backend.prepare(model, device)
             assert prepared_model is not None
 
-            # TODO after converting all npz files to protobuf, we can delete this.
-            for test_data_npz in glob.glob(os.path.join(model_dir, "test_data_*.npz")):
-                test_data = np.load(test_data_npz, encoding="bytes")
-                inputs = list(test_data["inputs"])
-                outputs = list(prepared_model.run(inputs))
-                ref_outputs = test_data["outputs"]
-                self.assert_similar_outputs(
-                    ref_outputs, outputs, rtol=model_test.rtol, atol=model_test.atol
-                )
+            if use_dummy:
+                # When the backend test goes through a test involving a
+                # model stored in onnx/backend/test/data/light,
+                # this function generates expected output coming from
+                # from ReferenceEvaluator run with random inputs.
+                # A couple of models include many Conv operators and the
+                # python implementation is slow (such as test_bvlc_alexnet).
+                with open(model_pb_path, "rb") as f:
+                    onx = onnx.load(f)
+
+                test_data_set = os.path.join(model_dir, "test_data_set_0")
+                if not os.path.exists(test_data_set):
+                    os.mkdir(test_data_set)
+                feeds = {}
+                inits = set(i.name for i in onx.graph.initializer)
+                n_input = 0
+                inputs = []
+                for i in range(len(onx.graph.input)):
+                    if onx.graph.input[i].name in inits:
+                        continue
+                    name = os.path.join(test_data_set, f"input_{n_input}.pb")
+                    inputs.append(name)
+                    n_input += 1
+                    x = onx.graph.input[i]
+                    value = self.generate_dummy_data(
+                        x, seed=0, name=model_test.model_name, random=False
+                    )
+                    feeds[x.name] = value
+                    with open(name, "wb") as f:
+                        f.write(onnx.numpy_helper.from_array(value).SerializeToString())
+
+                # loads expected output if any available
+                prefix = os.path.splitext(model_pb_path)[0]
+                expected_outputs = []
+                for i in range(len(onx.graph.output)):
+                    name = f"{prefix}_output_{i}.pb"
+                    if os.path.exists(name):
+                        expected_outputs.append(name)
+                        continue
+                    expected_outputs = None
+                    break
+
+                if expected_outputs is None:
+                    ref = onnx.reference.ReferenceEvaluator(onx)
+                    outputs = ref.run(None, feeds)
+                    for i, o in enumerate(outputs):
+                        name = os.path.join(test_data_set, f"output_{i}.pb")
+                        with open(name, "wb") as f:
+                            f.write(onnx.numpy_helper.from_array(o).SerializeToString())
+                else:
+                    for i, o in enumerate(expected_outputs):
+                        name = os.path.join(test_data_set, f"output_{i}.pb")
+                        shutil.copy(o, name)
+            else:
+                # TODO after converting all npz files to protobuf, we can delete this.
+                for test_data_npz in glob.glob(
+                    os.path.join(model_dir, "test_data_*.npz")
+                ):
+                    test_data = np.load(test_data_npz, encoding="bytes")
+                    inputs = list(test_data["inputs"])
+                    outputs = list(prepared_model.run(inputs))
+                    ref_outputs = test_data["outputs"]
+                    self.assert_similar_outputs(
+                        ref_outputs, outputs, rtol=model_test.rtol, atol=model_test.atol
+                    )
+
             for test_data_dir in glob.glob(os.path.join(model_dir, "test_data_set*")):
                 inputs = []
                 inputs_num = len(glob.glob(os.path.join(test_data_dir, "input_*.pb")))
@@ -382,7 +489,7 @@ class Runner:
             elif model_type_proto.HasField("optional_type"):
                 optional = onnx.OptionalProto()
                 optional.ParseFromString(protobuf_content)
-                target_list.append(numpy_helper.to_optional(optional))
+                target_list.append(numpy_helper.to_optional(optional))  # type: ignore[arg-type]
             else:
                 print(
                     "Loading proto of that specific type (Map/Sparse Tensor) is currently not supported"
