@@ -1,6 +1,6 @@
-/*
- * SPDX-License-Identifier: Apache-2.0
- */
+// Copyright (c) ONNX Project Contributors
+//
+// SPDX-License-Identifier: Apache-2.0
 
 #include "onnx/checker.h"
 #include "onnx/common/file_utils.h"
@@ -8,6 +8,7 @@
 #include "onnx/defs/schema.h"
 #include "onnx/defs/tensor_proto_util.h"
 #include "onnx/proto_utils.h"
+#include "onnx/shape_inference/implementation.h"
 #include "onnx/string_utils.h"
 
 #include <fstream>
@@ -16,6 +17,8 @@
 
 #ifdef _WIN32
 #include <direct.h>
+#include <filesystem>
+
 #else // POSIX
 #include <sys/stat.h>
 #endif
@@ -126,10 +129,68 @@ void check_tensor(const TensorProto& tensor, const CheckerContext& ctx) {
     for (const StringStringEntryProto& entry : tensor.external_data()) {
       if (entry.has_key() && entry.has_value() && entry.key() == "location") {
         has_location = true;
-        std::string data_path = path_join(ctx.get_model_dir(), entry.value());
-        // use stat to check whether the file exists
-        struct stat buffer;
+#ifdef _WIN32
+        auto file_path = std::filesystem::path(utf8str_to_wstring(entry.value()));
+        if (file_path.is_absolute()) {
+          fail_check(
+              "Location of external TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be a relative path, but it is an absolute path: ",
+              entry.value());
+        }
+        auto relative_path = file_path.lexically_normal().make_preferred().wstring();
+        // Check that normalized relative path contains ".." on Windows.
+        if (relative_path.find(L"..", 0) != std::string::npos) {
+          fail_check(
+              "Data of TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be file inside the ",
+              ctx.get_model_dir(),
+              ", but the '",
+              entry.value(),
+              "' points outside the directory");
+        }
+        std::wstring data_path = path_join(utf8str_to_wstring(ctx.get_model_dir()), relative_path);
+        struct _stat64 buff;
+        if (_wstat64(data_path.c_str(), &buff) != 0) {
+          fail_check(
+              "Data of TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be stored in ",
+              entry.value(),
+              ", but it doesn't exist or is not accessible.");
+        }
+#else // POSIX
+        if (entry.value().empty()) {
+          fail_check("Location of external TensorProto ( tensor name: ", tensor.name(), ") should not be empty.");
+        } else if (entry.value()[0] == '/') {
+          fail_check(
+              "Location of external TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be a relative path, but it is an absolute path: ",
+              entry.value());
+        }
+        std::string relative_path = clean_relative_path(entry.value());
+        // Check that normalized relative path contains ".." on POSIX
+        if (relative_path.find("..", 0) != std::string::npos) {
+          fail_check(
+              "Data of TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be file inside the ",
+              ctx.get_model_dir(),
+              ", but the '",
+              entry.value(),
+              "' points outside the directory");
+        }
+        std::string data_path = path_join(ctx.get_model_dir(), relative_path);
+        // use stat64 to check whether the file exists
+#if defined(__APPLE__) || defined(__wasm__)
+        struct stat buffer; // APPLE does not have stat64
         if (stat((data_path).c_str(), &buffer) != 0) {
+#else
+        struct stat64 buffer; // All POSIX except APPLE have stat64
+        if (stat64((data_path).c_str(), &buffer) != 0) {
+#endif
           fail_check(
               "Data of TensorProto ( tensor name: ",
               tensor.name(),
@@ -137,6 +198,16 @@ void check_tensor(const TensorProto& tensor, const CheckerContext& ctx) {
               data_path,
               ", but it doesn't exist or is not accessible.");
         }
+        // Do not allow symlinks or directories.
+        if (!S_ISREG(buffer.st_mode)) {
+          fail_check(
+              "Data of TensorProto ( tensor name: ",
+              tensor.name(),
+              ") should be stored in ",
+              data_path,
+              ", but it is not regular file.");
+        }
+#endif
       }
     }
     if (!has_location) {
@@ -191,6 +262,10 @@ void check_tensor(const TensorProto& tensor, const CheckerContext& ctx) {
       case TensorProto::BOOL:
       case TensorProto::FLOAT16:
       case TensorProto::BFLOAT16:
+      case TensorProto::FLOAT8E4M3FN:
+      case TensorProto::FLOAT8E4M3FNUZ:
+      case TensorProto::FLOAT8E5M2:
+      case TensorProto::FLOAT8E5M2FNUZ:
         check_field(int32_data);
         break;
 
@@ -529,6 +604,18 @@ void check_attribute(const AttributeProto& attr, const CheckerContext& ctx, cons
   }
 }
 
+void print_warning_if_has_experimental(const std::unordered_set<std::string>& used_experimental_ops) {
+  if (!used_experimental_ops.empty()) {
+    std::string all_experimental_ops;
+    for (const auto& op : used_experimental_ops) {
+      all_experimental_ops += " " + op + ",";
+    }
+    // Remove the last comma which is unnecessary
+    all_experimental_ops.pop_back();
+    std::cout << "Warning: Model contains experimental ops:" + all_experimental_ops << std::endl;
+  }
+}
+
 void check_node(const NodeProto& node, const CheckerContext& ctx, const LexicalScopeContext& lex_ctx) {
   enforce_non_empty_field(node, op_type);
 
@@ -536,12 +623,6 @@ void check_node(const NodeProto& node, const CheckerContext& ctx, const LexicalS
     fail_check("NodeProto (name: ", node.name(), ", type: ", node.op_type(), ") has zero input and zero output.");
   }
 
-  // If encounter experimental op, stop checking
-  if (check_is_experimental_op(node.op_type())) {
-    std::cerr << "Warning: Checker does not support models with experimental ops: " << node.op_type() << std::endl;
-    return;
-  }
-
   // Resolve domain for node
   const auto& opset_imports = ctx.get_opset_imports();
   auto dit = opset_imports.find(node.domain());
@@ -550,10 +631,22 @@ void check_node(const NodeProto& node, const CheckerContext& ctx, const LexicalS
   }
   auto domain_version = dit->second;
 
+  // for ops referencing local functions, there is no schema to verify it.
+  // will add a check to verify consistency between these ops and local functions.
+  std::unordered_set<std::string> seen_attr_names{};
   for (const auto& attr : node.attribute()) {
+    if (!seen_attr_names.insert(attr.name()).second) {
+      fail_check("Attribute '", attr.name(), "' appeared multiple times.");
+    };
+
     check_attribute(attr, ctx, lex_ctx);
   }
 
+  // This issue will be caught by check_graph instead
+  if (check_is_experimental_op(node)) {
+    return;
+  }
+
   const auto* schema = ctx.get_schema_registry()->GetSchema(node.op_type(), domain_version, node.domain());
   if (!schema) {
     if (node.domain() == ONNX_DOMAIN || node.domain() == AI_ONNX_ML_DOMAIN || node.domain() == "ai.onnx" ||
@@ -646,7 +739,7 @@ void check_graph(const GraphProto& graph, const CheckerContext& ctx, const Lexic
     check_sparse_tensor(sparse_init, ctx);
     lex_ctx.add(name);
   }
-
+  std::unordered_set<std::string> used_experimental_ops;
   for (const auto& node : graph.node()) {
     // nodes must be in topologically sorted order
     for (const auto& input : node.input()) {
@@ -667,6 +760,10 @@ void check_graph(const GraphProto& graph, const CheckerContext& ctx, const Lexic
       }
     }
 
+    if (check_is_experimental_op(node)) {
+      used_experimental_ops.insert(node.op_type());
+    }
+
     // This needs to happen before SSA check since we don't want to recurse and
     // find that outputs from control flow ops are colliding with names in the
     // inner block
@@ -696,6 +793,7 @@ void check_graph(const GraphProto& graph, const CheckerContext& ctx, const Lexic
       lex_ctx.add(output);
     }
   }
+  print_warning_if_has_experimental(used_experimental_ops);
 }
 
 // Utilify function to get the imported version of domain from opset imports
@@ -749,7 +847,7 @@ void check_opset_compatibility(
     fail_check(
         "Opset import for domain " + node.domain() + " in function op " + node.op_type() +
         "is not compatible with the version imported by model. FunctionOp imports version " +
-        ONNX_NAMESPACE::to_string(func_opset_version) + "whereas model imports version " +
+        ONNX_NAMESPACE::to_string(func_opset_version) + " whereas model imports version " +
         ONNX_NAMESPACE::to_string(model_opset_version));
   }
 }
@@ -826,7 +924,7 @@ void check_function(const FunctionProto& function, const CheckerContext& ctx, co
       fail_check("function (", function.name(), ") should not have duplicate attributes specified.");
     }
   }
-
+  std::unordered_set<std::string> used_experimental_ops;
   for (const auto& node : function.node()) {
     // nodes must be in topologically sorted order
     for (const auto& input : node.input()) {
@@ -850,7 +948,9 @@ void check_function(const FunctionProto& function, const CheckerContext& ctx, co
     // check whether the opset version imported for a domain by function and model are
     // compatible
     check_opset_compatibility(node, ctx_copy, func_opset_imports, model_opset_imports);
-
+    if (check_is_experimental_op(node)) {
+      used_experimental_ops.insert(node.op_type());
+    }
     check_node(node, ctx_copy, lex_ctx);
 
     // check for SSA form
@@ -868,6 +968,7 @@ void check_function(const FunctionProto& function, const CheckerContext& ctx, co
       lex_ctx.add(output);
     }
   }
+  print_warning_if_has_experimental(used_experimental_ops);
 }
 
 void check_model(const ModelProto& model, CheckerContext& ctx) {
@@ -875,7 +976,7 @@ void check_model(const ModelProto& model, CheckerContext& ctx) {
     fail_check("The model does not have an ir_version set properly.");
   }
   if (model.ir_version() > IR_VERSION) {
-    fail_check("Your model ir_version is higher than the checker's.");
+    fail_check("Your model ir_version ", model.ir_version(), " is higher than the checker's (", IR_VERSION, ").");
   }
   if (model.metadata_props_size() > 1) {
     std::unordered_set<std::string> keys;
@@ -909,10 +1010,11 @@ void check_model(const ModelProto& model, CheckerContext& ctx) {
 
   if (ctx.get_ir_version() >= 0x00000008) {
     check_model_local_functions(model, ctx, lex_ctx);
+    // TODO: check consistency between local functions and ops referencing it.
   }
 }
 
-void check_model(const std::string& model_path) {
+void check_model(const std::string& model_path, bool full_check) {
   ModelProto model;
   LoadProtoFromPath(model_path, model);
 
@@ -924,11 +1026,23 @@ void check_model(const std::string& model_path) {
   }
   ctx.set_model_dir(model_dir);
   check_model(model, ctx);
+
+  if (full_check) {
+    ShapeInferenceOptions options{true, 1, false};
+    ONNX_NAMESPACE::shape_inference::InferShapes(model, ctx.get_schema_registry(), options);
+  }
 }
 
-void check_model(const ModelProto& model) {
+void check_model(const ModelProto& model, bool full_check) {
   CheckerContext ctx;
   check_model(model, ctx);
+  if (full_check) {
+    ShapeInferenceOptions options{true, 1, false};
+    // Do not update the model in place by the check from shape inference
+    // because checker should not modify the original model
+    ModelProto copy = model;
+    ONNX_NAMESPACE::shape_inference::InferShapes(copy, ctx.get_schema_registry(), options);
+  }
 }
 
 std::set<std::string> experimental_ops = {
@@ -944,8 +1058,8 @@ std::set<std::string> experimental_ops = {
     "Scale",
     "ScaledTanh"};
 
-bool check_is_experimental_op(std::string node_op_type) {
-  return (experimental_ops.count(node_op_type)) ? true : false;
+bool check_is_experimental_op(const NodeProto& node) {
+  return (node.domain() == ONNX_DOMAIN || node.domain() == "ai.onnx") && experimental_ops.count(node.op_type());
 }
 
 #undef fail_check
