@@ -36,7 +36,7 @@ class FunctionBodyHelper {
     }
 
     template <typename T>
-    AttributeProtoWrapper(const std::string& attr_name, T value) {
+    AttributeProtoWrapper(const std::string& attr_name, const T& value) {
       proto = MakeAttribute(attr_name, value);
     }
   };
@@ -142,7 +142,7 @@ class FunctionBuilder {
   }
 
   template <typename T>
-  FunctionBuilder& Add(const char* node_txt, const std::string& attr_name, T attr_value) {
+  FunctionBuilder& Add(const char* node_txt, const std::string& attr_name, const T& attr_value) {
     return Add(node_txt, MakeAttribute(attr_name, attr_value));
   }
 
