@@ -1,5 +1,6 @@
-// Copyright (c) ONNX Project Contributors.
-// Licensed under the MIT license.
+/*
+ * SPDX-License-Identifier: Apache-2.0
+ */
 
 #include <algorithm>
 #include <cmath>
@@ -109,7 +110,7 @@ W --> Gemm --> Y --> Loss --> O
 The tensors named in attributes "xs", "zs", and "y" define the differentiated
 computation graph, and the inputs to Gradient node define the values at
 which the gradient is computed. We can feed different tensors to the identified
-graph. For example, one can compute the gradient of Y with respect to H at 
+graph. For example, one can compute the gradient of Y with respect to H at
 a specific value of H, H_1, by providing that value as an input to the Gradient
 node.
 
@@ -185,157 +186,24 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "\"zs\" are the minimal independent variable set that determines "
             "the value of \"y\".",
             AttributeProto::STRING)
-        .TypeConstraint(
-            "T1",
-            OpSchema::all_tensor_types(),
-            "Allow outputs to be any kind of tensor.")
+        .TypeConstraint("T1", OpSchema::all_tensor_types(), "Allow outputs to be any kind of tensor.")
         .TypeConstraint(
             "T2",
-            {"tensor(float16)",
-             "tensor(float)",
-             "tensor(double)"},
+            {"tensor(float16)", "tensor(float)", "tensor(double)"},
             "Allow inputs to be any kind of floating-point tensor."));
 
-
-static const char* GraphCall_ver1_doc = R"DOC(
-The GraphCall operator invokes a graph inside TrainingInfoProto's
-algorithm field. The GraphCall inputs and outputs are bound to those of
-invoked graph by position. If a graph input has an initializer, that input
-is considered optional. All graph outputs are optional.
-
-Below Python syntax is used for describing dictionary and list.
-
-Assume that ModelProto's graph field has
-- name: "MyInferenceGraph"
-- input: ["X", "W", "Z"]
-- initializer: [W]
-- output: ["Y"]
-
-as visualized below for inference.
-
-```
-X -----.
-       |
-       v
-W --> Conv --> H --> Gemm --> Y
-                      ^
-                      |
-                      Z
-```
-
-Assume that the training algorithm contains
-
-- inputs: ["X_1", "Z_1", "C"]
-- initializer: [T]
-- outputs: ["W_new"]
-
-with a dictionary
-
-- update_binding: {"W": "W_new", "T": "T_new"}
-
-Inside the training algorithm graph, one can invoke the inference
-graph via adding a GraphCall node with
-
-- inputs: ["X_1", "W", Z_1"]
-- outputs: ["Y_1"]
-- an attribute graph_name="MyInferenceGraph",
-
-The initializers, "W" and "T" in this case, in update_binding
-are considered globally-visible and mutable variables, which
-can be used as inputs of operators in the training graph.
-
-An example training algorithm graph may look like
-
-```
-.-------- W (a global and mutable variable from
-|         |  the inference graph)
-|         |
-|   .-----'-----------.
-|   |                 |
-|   |                 v
-|   | .-- X_1 --> GraphCall(graph_name="MyInferenceGraph")
-|   | |            |  |
-|   | |            |  |
-|   | |   Z_1 -----'  |
-|   | |    |          V
-|   | |    |         Y_1 ---> Loss ---> O
-|   | |    |                    ^
-|   | |    |                    |
-|   | `--. |                    C
-|   |    | |                    |
-|   |    | |   .----------------'
-|   |    | |   |
-|   |    v v   v
-|   `--> Gradient(xs=["W"], zs=["X_1", "Z_1", "C"], y="O")
-|        |
-|        v
-|      dO_dW (gradient of W)      1 (a scalar one)
-|        |                        |
-|        V                        v
-|       Div <--- T ------------> Add ---> T_new
-|        |    (T is the number of training iterations.
-|        |     T is also globally visible and mutable.)
-|        v
-`-----> Sub ----> W_new
-```
-
-where Loss is a dummy node which computes the minimized objective function.
-
-The variable "W" is an optional input in the called graph.
-If the user omits it, the input list of GraphCall becomes ["X_1", "", "Z_1"].
-In this case, from the view of computation graph, the Conv operator invoked by
-GraphCall's may be still connected the global "W" variable and therefore the
-structure of the computation graph is unchanged.
-)DOC";
-
-ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
-    GraphCall,
-    1,
-    OpSchema()
-        .SetDoc(GraphCall_ver1_doc)
-        .Input(
-            0,
-            "Inputs",
-            "Inputs fed to the invoked graph. "
-            "The i-th input here goes to the i-th input of the invoked graph. "
-            "To omit an optional input in this field, "
-            "the user can drop it or use an empty string.",
-            "T",
-            OpSchema::Variadic,
-            false)
-        .Output(
-            0,
-            "Outputs",
-            "The outputs generated by the called graph. "
-            "Its i-th value is bound to the i-th output of the called graph. "
-            "Similar to the inputs, all outputs are optional.",
-            "T",
-            OpSchema::Variadic,
-            false)
-        .Attr(
-            "graph_name",
-            "The invoked graph's name. "
-            "The only allowed value is the name of the inference graph, "
-            "which is stored in \"ModelProto.graph.name\" "
-            "in the ONNX model format.",
-            AttributeProto::STRING)
-        .TypeConstraint(
-            "T",
-            OpSchema::all_tensor_types(),
-            "Allow inputs and outputs to be any kind of tensor."));
-
 static const char* Adagrad_ver1_doc = R"DOC(
     Compute one iteration of ADAGRAD, a stochastic gradient based optimization
     algorithm. This operator can conduct the optimization of multiple tensor variables.
 
     Let's define the behavior of this operator. As you can imagine, ADAGRAD requires
     some parameters:
-     
+
      - The initial learning-rate "R".
      - The update count "T". That is, the number of training iterations conducted.
      - A L2-norm regularization coefficient "norm_coefficient".
      - A learning-rate decay factor "decay_factor".
-     - A small constant "epsilon" to avoid dividing-by-zero. 
+     - A small constant "epsilon" to avoid dividing-by-zero.
 
     At each ADAGRAD iteration, the optimized tensors are moved along a direction
     computed based on their estimated gradient and accumulated squared gradient. Assume
@@ -389,7 +257,8 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "inputs",
             "The current values of optimized tensors, followed by their "
             "respective gradients, followed by their respective accumulated squared gradients."
-            "For example, if two tensor \"X_1\" and \"X_2\" " "are optimized, "
+            "For example, if two tensor \"X_1\" and \"X_2\" "
+            "are optimized, "
             "The input list would be "
             "[\"X_1\", \"X_2\", "
             "gradient of \"X_1\", "
@@ -410,11 +279,7 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "T3",
             OpSchema::Variadic,
             false)
-        .Attr(
-            "epsilon",
-            "Small scalar to avoid dividing by zero.",
-            AttributeProto::FLOAT,
-            1e-6f)
+        .Attr("epsilon", "Small scalar to avoid dividing by zero.", AttributeProto::FLOAT, 1e-6f)
         .Attr(
             "decay_factor",
             "The decay factor of learning rate after one update."
@@ -428,38 +293,30 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "which means no regularization.",
             AttributeProto::FLOAT,
             0.0f)
-        .TypeConstraint(
-            "T1",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input types to float scalars.")
-        .TypeConstraint(
-            "T2",
-            {"tensor(int64)"},
-            "Constrain input types to 64-bit integer scalars.")
-        .TypeConstraint(
-            "T3",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input and output types to float tensors.")
+        .TypeConstraint("T1", {"tensor(float)", "tensor(double)"}, "Constrain input types to float scalars.")
+        .TypeConstraint("T2", {"tensor(int64)"}, "Constrain input types to 64-bit integer scalars.")
+        .TypeConstraint("T3", {"tensor(float)", "tensor(double)"}, "Constrain input and output types to float tensors.")
         .TypeAndShapeInferenceFunction([](InferenceContext& ctx) {
-            // In comments below, we assume that the input list is
-            // [R, T, X1, X2, G1, G2, H1, H2] and the output list is
-            // [X1_new, X2_new, H1_new, H2_new].
+          // In comments below, we assume that the input list is
+          // [R, T, X1, X2, G1, G2, H1, H2] and the output list is
+          // [X1_new, X2_new, H1_new, H2_new].
 
-            // Compute the number of tuples (X, G, H).
-            auto num_optimized_tensors = (ctx.getNumInputs() - 2) / 3;
-            for (size_t i = 0; i < num_optimized_tensors; ++i) {
-              // Pass X1's and X2's shapes to X1_new and X2_new, respectively.
-              size_t i_in = 2 + i;
-              size_t i_out = i;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
+          // Compute the number of tuples (X, G, H).
+          auto num_optimized_tensors = (ctx.getNumInputs() - 2) / 3;
+          for (size_t i = 0; i < num_optimized_tensors; ++i) {
+            // Pass X1's and X2's shapes to X1_new and X2_new, respectively.
+            size_t i_in = 2 + i;
+            size_t i_out = i;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
 
-              // Pass H1's and H2's shapes to H1_new and H2_new, respectively.
-              i_in = 2 + 2 * num_optimized_tensors + i;
-              i_out = i + num_optimized_tensors;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
-            }}));
+            // Pass H1's and H2's shapes to H1_new and H2_new, respectively.
+            i_in = 2 + 2 * num_optimized_tensors + i;
+            i_out = i + num_optimized_tensors;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
+          }
+        }));
 
 static const char* Momentum_ver1_doc = R"DOC(
     Compute one iteration of stochastic gradient update with momentum.
@@ -467,7 +324,7 @@ static const char* Momentum_ver1_doc = R"DOC(
 
     Let's define the behavior of this operator. As you can imagine, SG with momentum requires
     several parameters:
-     
+
      - The learning-rate "R".
      - The update count "T". That is, the number of conducted training iterations. It should
        be zero in the first training iteration.
@@ -481,7 +338,7 @@ static const char* Momentum_ver1_doc = R"DOC(
     Other necessary inputs are "X"'s gradient (called "G") and "X"'s momentum (called "V"). This
     Momentum operator maps all these inputs to the new value of "X" (called "X_new") and its new
     momentum (called "V_new").
-    
+
     This operator supports two different momentum algorithms. Set the attribute "mode" to
     "nesterov" if Nesterov's momentum is desired. Otherwise, set the attribute "model" to
     "standard" to use standard momentum. Computation details are described subsequently.
@@ -551,64 +408,50 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "T3",
             OpSchema::Variadic,
             false)
-        .Attr(
-            "alpha",
-            "The decay factor of momentum. It should be a scalar.",
-            AttributeProto::FLOAT)
+        .Attr("alpha", "The decay factor of momentum. It should be a scalar.", AttributeProto::FLOAT)
         .Attr(
             "beta",
             "The coefficient of gradient in computing new momentum. It should be a scalar.",
             AttributeProto::FLOAT)
-        .Attr(
-            "norm_coefficient",
-            "Coefficient of 0.5 * norm_coefficient * ||X||^2.",
-            AttributeProto::FLOAT)
+        .Attr("norm_coefficient", "Coefficient of 0.5 * norm_coefficient * ||X||^2.", AttributeProto::FLOAT)
         .Attr(
             "mode",
             "Its value should be either \"nesterov\" or \"standard\". The value \"nesterov\" leads "
             "to the use of Nesterov's momentum while \"standard\" invokes stochastic gradient method "
             "using standard momentum",
             AttributeProto::STRING)
-        .TypeConstraint(
-            "T1",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input types to float scalars.")
-        .TypeConstraint(
-            "T2",
-            {"tensor(int64)"},
-            "Constrain input types to 64-bit integer scalars.")
-        .TypeConstraint(
-            "T3",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input types to float tensors.")
+        .TypeConstraint("T1", {"tensor(float)", "tensor(double)"}, "Constrain input types to float scalars.")
+        .TypeConstraint("T2", {"tensor(int64)"}, "Constrain input types to 64-bit integer scalars.")
+        .TypeConstraint("T3", {"tensor(float)", "tensor(double)"}, "Constrain input types to float tensors.")
         .TypeAndShapeInferenceFunction([](InferenceContext& ctx) {
-            // Assume that the input list is [R, T, X1, X2, G1, G2, V1, V2] and
-            // output list is [X1_new, X2_new, V1_new, V2_new] for explaining
-            // the code below in a simpler way.
+          // Assume that the input list is [R, T, X1, X2, G1, G2, V1, V2] and
+          // output list is [X1_new, X2_new, V1_new, V2_new] for explaining
+          // the code below in a simpler way.
 
-            // The count of input tensors excluding "R" and "T".
-            auto num_adjustable_tensors = ctx.getNumInputs() - 2;
+          // The count of input tensors excluding "R" and "T".
+          auto num_adjustable_tensors = ctx.getNumInputs() - 2;
 
-            // Check number of (optimized tensor, gradient, momentum) tuples.
-            if (num_adjustable_tensors % 3 != 0)
-              fail_shape_inference(
-                  "The sum of optimized tensor count and momentum tensor count ",
-                  "should be a multiple of 2 in the input list of Momentum operator");
+          // Check number of (optimized tensor, gradient, momentum) tuples.
+          if (num_adjustable_tensors % 3 != 0) {
+            fail_shape_inference(
+                "The sum of optimized tensor count and momentum tensor count ",
+                "should be a multiple of 2 in the input list of Momentum operator");
+          }
 
-            // The count of "X1" and "X2".
-            auto num_optimized_tensors = num_adjustable_tensors / 3;
-            for (size_t i = 0; i < num_optimized_tensors; ++i){
-              // Pass X1's/X2's shapes to X1_new/X2_new.
-              size_t i_in = 2 + i;
-              size_t i_out = i;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
-              // Pass V1's/V2's shapes to V1_new/V2_new.
-              i_in = 2 + 2 * num_optimized_tensors + i;
-              i_out = i + num_optimized_tensors;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
-            }
+          // The count of "X1" and "X2".
+          auto num_optimized_tensors = num_adjustable_tensors / 3;
+          for (size_t i = 0; i < num_optimized_tensors; ++i) {
+            // Pass X1's/X2's shapes to X1_new/X2_new.
+            size_t i_in = 2 + i;
+            size_t i_out = i;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
+            // Pass V1's/V2's shapes to V1_new/V2_new.
+            i_in = 2 + 2 * num_optimized_tensors + i;
+            i_out = i + num_optimized_tensors;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
+          }
         }));
 
 static const char* Adam_ver1_doc = R"DOC(
@@ -617,18 +460,18 @@ static const char* Adam_ver1_doc = R"DOC(
 
     Let's define the behavior of this operator. First of all, Adam requires
     some parameters:
-     
+
      - The learning-rate "R".
      - The update count "T". That is, the number of training iterations conducted.
      - A L2-norm regularization coefficient "norm_coefficient".
-     - A small constant "epsilon" to avoid dividing-by-zero. 
+     - A small constant "epsilon" to avoid dividing-by-zero.
      - Two coefficients, "alpha" and "beta".
 
     At each Adam iteration, the optimized tensors are moved along a direction
     computed based on their exponentially-averaged historical gradient and
     exponentially-averaged historical squared gradient. Assume that only a tensor
     "X" is being optimized. The rest of required information is
-    
+
      - the value of "X",
      - "X"'s gradient (denoted by "G"),
      - "X"'s exponentially-averaged historical gradient (denoted by "V"), and
@@ -638,8 +481,8 @@ static const char* Adam_ver1_doc = R"DOC(
     are stored as this operator's attributes. Specifically, this operator's input tensor
     list is ["R", "T", "X", "G", "V", "H"]. That is, "R" is the first input, "T" is
     the second input, and so on. Other parameters are given as attributes because they
-    are constants. Moreover, the corresponding output tensors are 
-    
+    are constants. Moreover, the corresponding output tensors are
+
      - the new value of "X" (called "X_new"),
      - the new exponentially-averaged historical gradient (denoted by "V_new"), and
      - the new exponentially-averaged historical squared gradient (denoted by "H_new").
@@ -669,7 +512,7 @@ static const char* Adam_ver1_doc = R"DOC(
       X_new = X - R_adjusted * V_new / H_sqrt
 
       // Post-update regularization.
-      X_final = (1 - norm_coefficient_post) * X_new 
+      X_final = (1 - norm_coefficient_post) * X_new
 
     If there are multiple inputs to be optimized, the pseudo code will be applied
     independently to each of them.
@@ -734,58 +577,47 @@ ONNX_PREVIEW_TRAINING_OPERATOR_SET_SCHEMA(
             "which means no regularization.",
             AttributeProto::FLOAT,
             0.0f)
-        .Attr(
-            "epsilon",
-            "Small scalar to avoid dividing by zero.",
-            AttributeProto::FLOAT,
-            1e-6f)
-        .TypeConstraint(
-            "T1",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input types to float scalars.")
-        .TypeConstraint(
-            "T2",
-            {"tensor(int64)"},
-            "Constrain input types to 64-bit integer scalars.")
-        .TypeConstraint(
-            "T3",
-            {"tensor(float)", "tensor(double)"},
-            "Constrain input and output types to float tensors.")
-        .TypeAndShapeInferenceFunction([](InferenceContext &ctx) {
-            // Assume that the input list is [R, T, X1, X2, G1, G2, V1, V2, H1, H2] and
-            // output list is [X1_new, X2_new, V1_new, V2_new, H1_new, H2_new] for explaining
-            // the code below in a simpler way.
+        .Attr("epsilon", "Small scalar to avoid dividing by zero.", AttributeProto::FLOAT, 1e-6f)
+        .TypeConstraint("T1", {"tensor(float)", "tensor(double)"}, "Constrain input types to float scalars.")
+        .TypeConstraint("T2", {"tensor(int64)"}, "Constrain input types to 64-bit integer scalars.")
+        .TypeConstraint("T3", {"tensor(float)", "tensor(double)"}, "Constrain input and output types to float tensors.")
+        .TypeAndShapeInferenceFunction([](InferenceContext& ctx) {
+          // Assume that the input list is [R, T, X1, X2, G1, G2, V1, V2, H1, H2] and
+          // output list is [X1_new, X2_new, V1_new, V2_new, H1_new, H2_new] for explaining
+          // the code below in a simpler way.
 
-            // The count of input tensors excluding "R" and "T".
-            auto num_adjustable_tensors = ctx.getNumInputs() - 2;
+          // The count of input tensors excluding "R" and "T".
+          auto num_adjustable_tensors = ctx.getNumInputs() - 2;
 
-            // Check number of (optimized tensor, gradient, momentum) tuples.
-            if (num_adjustable_tensors % 4 != 0)
-              fail_shape_inference(
-                  "The sum of optimized tensor count, gradient tensor count, momentum tensor count, ",
-                  "accumulated squared-gradient tensor count should be a multiple of 4 in the ",
-                  "\"inputs\" of Adam operator.");
+          // Check number of (optimized tensor, gradient, momentum) tuples.
+          if (num_adjustable_tensors % 4 != 0) {
+            fail_shape_inference(
+                "The sum of optimized tensor count, gradient tensor count, momentum tensor count, ",
+                "accumulated squared-gradient tensor count should be a multiple of 4 in the ",
+                "\"inputs\" of Adam operator.");
+          }
 
-            // The count of "X1" and "X2".
-            auto num_optimized_tensors = num_adjustable_tensors / 4;
-            for (size_t i = 0; i < num_optimized_tensors; ++i){
-              // Pass X1's/X2's shapes to X1_new/X2_new.
-              size_t i_in = 2 + i;
-              size_t i_out = i;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
+          // The count of "X1" and "X2".
+          auto num_optimized_tensors = num_adjustable_tensors / 4;
+          for (size_t i = 0; i < num_optimized_tensors; ++i) {
+            // Pass X1's/X2's shapes to X1_new/X2_new.
+            size_t i_in = 2 + i;
+            size_t i_out = i;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
 
-              // Pass V1's/V2's shapes to V1_new/V2_new.
-              i_in = 2 + 2 * num_optimized_tensors + i;
-              i_out = num_optimized_tensors + i;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
+            // Pass V1's/V2's shapes to V1_new/V2_new.
+            i_in = 2 + 2 * num_optimized_tensors + i;
+            i_out = num_optimized_tensors + i;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
 
-              // Pass H1's/H2's shapes to H1_new/H2_new.
-              i_in = 2 + 3 * num_optimized_tensors + i;
-              i_out = 2 * num_optimized_tensors + i;
-              propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
-              propagateShapeFromInputToOutput(ctx, i_in, i_out);
-        }}));
+            // Pass H1's/H2's shapes to H1_new/H2_new.
+            i_in = 2 + 3 * num_optimized_tensors + i;
+            i_out = 2 * num_optimized_tensors + i;
+            propagateElemTypeFromInputToOutput(ctx, i_in, i_out);
+            propagateShapeFromInputToOutput(ctx, i_in, i_out);
+          }
+        }));
 
 } // namespace ONNX_NAMESPACE
