@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 # pylint: disable=W0221
 
@@ -8,4 +10,6 @@ from onnx.reference.op_run import OpRun
 
 class Selu(OpRun):
     def _run(self, x, alpha=None, gamma=None):  # type: ignore
-        return (np.where(x > 0, x, np.exp(x) * alpha - alpha) * gamma,)
+        return (
+            (np.where(x > 0, x, np.exp(x) * alpha - alpha) * gamma).astype(x.dtype),
+        )
