@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 # pylint: disable=W0221
 
@@ -5,7 +7,6 @@ from typing import Optional, Tuple
 
 import numpy as np
 
-from onnx.defs import onnx_opset_version
 from onnx.reference.op_run import OpRun
 
 
@@ -36,9 +37,3 @@ class Shape_15(Shape_1):
         if ab is None:
             return (np.array(data.shape, dtype=np.int64),)
         return (np.array(data.shape[ab[0] : ab[1]], dtype=np.int64),)
-
-
-if onnx_opset_version() >= 15:
-    Shape = Shape_15
-else:
-    Shape = Shape_1  # type: ignore
