@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 # pylint: disable=W0221
 
@@ -27,5 +29,5 @@ class SpaceToDepth(OpRun):
             H // blocksize,
             W // blocksize,
         )
-        y = np.reshape(transposed, finalshape)
+        y = np.reshape(transposed, finalshape).astype(data.dtype)
         return (y,)
