@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 # pylint: disable=W0221
 
@@ -5,11 +7,11 @@ from warnings import catch_warnings, simplefilter
 
 import numpy as np
 
-from ._op import OpRunUnaryNum
+from onnx.reference.ops._op import OpRunUnaryNum
 
 
 class Sqrt(OpRunUnaryNum):
     def _run(self, x):  # type: ignore
         with catch_warnings():
             simplefilter("ignore")
-            return (np.sqrt(x),)
+            return (np.sqrt(x).astype(x.dtype),)
