@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 # pylint: disable=W0221
 
@@ -6,4 +8,4 @@ from onnx.reference.op_run import OpRun
 
 class Sum(OpRun):
     def _run(self, *args):  # type: ignore
-        return (sum(args),)
+        return (sum(args).astype(args[0].dtype),)
