@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 import string
 import unittest
@@ -37,7 +39,6 @@ class TestAutomaticUpgrade(unittest.TestCase):
         if attrs is None:
             attrs = {}
 
-        global tested_ops
         tested_ops.append(op)
 
         n_inputs = len(input_shapes)
@@ -329,6 +330,14 @@ class TestAutomaticUpgrade(unittest.TestCase):
             "ConvTranspose", 1, [[1, 1, 5, 5], [1, 1, 3, 3]], [[1, 1, 7, 7]]
         )
 
+    def test_DeformConv(self) -> None:
+        self._test_op_upgrade(
+            "DeformConv",
+            19,
+            [[1, 1, 3, 3], [1, 1, 2, 2], [1, 8, 2, 2]],
+            [[1, 1, 2, 2]],
+        )
+
     def test_Cosh(self) -> None:
         self._test_op_upgrade("Cosh", 9)
 
@@ -1707,7 +1716,7 @@ class TestAutomaticUpgrade(unittest.TestCase):
         )
 
     def test_CenterCropPad(self) -> None:
-        input = helper.make_tensor(
+        input_ = helper.make_tensor(
             "input",
             TensorProto.FLOAT,
             dims=[2, 4],
@@ -1722,7 +1731,7 @@ class TestAutomaticUpgrade(unittest.TestCase):
             [[], []],
             [[3, 3]],
             [TensorProto.FLOAT, TensorProto.INT64],
-            initializer=[input, shape],
+            initializer=[input_, shape],
         )
 
     def test_BitwiseNot(self) -> None:
