@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 import unittest
 from typing import Sequence
@@ -5,7 +7,6 @@ from typing import Sequence
 import numpy as np
 
 import onnx.defs
-import onnx.onnx_cpp2py_export.checker as C
 from onnx import (
     GraphProto,
     SparseTensorProto,
@@ -66,8 +67,33 @@ class TestChecker(unittest.TestCase):
         node = helper.make_node("Relu", [""], ["Y"], name="test")
         self.assertRaises(checker.ValidationError, checker.check_node, node)
 
+    def test_check_function_nested(self) -> None:
+        func_domain = "local"
+        func_nested_opset_imports = [
+            helper.make_opsetid("", 14),
+            helper.make_opsetid(func_domain, 1),
+        ]
+        # nested identity/add function
+        func_nested_identity_add_name = "func_nested_identity_add"
+        func_nested_identity_add_inputs = ["a", "b"]
+        func_nested_identity_add_outputs = ["c"]
+        func_nested_identity_add_nodes = [
+            helper.make_node("func_identity", ["a"], ["a1"], domain=func_domain),
+            helper.make_node("func_identity", ["b"], ["b1"], domain=func_domain),
+            helper.make_node("func_add", ["a1", "b1"], ["c"], domain=func_domain),
+        ]
+        func_nested_identity_add = helper.make_function(
+            func_domain,
+            func_nested_identity_add_name,
+            func_nested_identity_add_inputs,
+            func_nested_identity_add_outputs,
+            func_nested_identity_add_nodes,
+            func_nested_opset_imports,
+        )
+        checker.check_function(func_nested_identity_add)
+
     def test_check_graph_ir_version_3(self) -> None:
-        ctx = C.CheckerContext()
+        ctx = checker.C.CheckerContext()
         ctx.ir_version = 3
         ctx.opset_imports = {"": onnx.defs.onnx_opset_version()}
 
