@@ -1,4 +1,8 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
+
+# pylint: disable=protected-access
 import glob
 import os
 import unittest
@@ -6,8 +10,7 @@ from os.path import join
 
 import pytest
 
-import onnx.hub as hub
-from onnx import ModelProto
+from onnx import ModelProto, hub
 
 
 @pytest.mark.skipif(
@@ -95,6 +98,21 @@ class TestModelHub(unittest.TestCase):
             AssertionError, lambda: hub.get_model_info("mnist", self.repo, opset=-1)
         )
 
+    def test_download_model_with_test_data(self) -> None:
+        directory = hub.download_model_with_test_data("mnist")
+        files = os.listdir(directory)
+        self.assertIsInstance(directory, str)
+        self.assertIn(member="model.onnx", container=files, msg="Onnx model not found")
+        self.assertIn(
+            member="test_data_set_0", container=files, msg="Test data not found"
+        )
+
+    def test_model_with_preprocessing(self) -> None:
+        model = hub.load_composite_model(
+            "ResNet50-fp32", preprocessing_model="ResNet-preproc"
+        )
+        self.assertIsInstance(model, ModelProto)
+
 
 if __name__ == "__main__":
     unittest.main()
