@@ -1,3 +1,5 @@
+# Copyright (c) ONNX Project Contributors
+
 # SPDX-License-Identifier: Apache-2.0
 import unittest
 
@@ -5,12 +7,10 @@ from onnx import defs, helper
 
 
 class TestRelu(unittest.TestCase):
-
     def test_relu(self) -> None:
-        self.assertTrue(defs.has('Relu'))
-        helper.make_node(
-            'Relu', ['X'], ['Y'])
+        self.assertTrue(defs.has("Relu"))
+        helper.make_node("Relu", ["X"], ["Y"])
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
